@@ -27,7 +27,9 @@
 #include <string.h>
 #include <stdio.h>
 
+#define M64P_CORE_PROTOTYPES 1
 #include "m64p_types.h"
+#include "m64p_config.h"
 #include "config.h"
 #include "callbacks.h"
 
@@ -56,11 +58,15 @@ typedef struct _config_section {
   struct _config_section *next;
   } config_section;
 
+typedef config_section *config_list;
+
 /* local variables */
-static int      l_ConfigInit = 0;
-static int      l_SaveConfigOnExit = 0;
-static char    *l_DataDirOverride = NULL;
-static config_section *l_SectionHead = NULL;
+static int         l_ConfigInit = 0;
+static int         l_SaveConfigOnExit = 0;
+static char       *l_DataDirOverride = NULL;
+static char       *l_ConfigDirOverride = NULL;
+static config_list l_ConfigListActive = NULL;
+static config_list l_ConfigListSaved = NULL;
 
 /* --------------- */
 /* local functions */
@@ -147,12 +153,209 @@ static void append_var_to_section(config_section *section, config_var *var)
     last_var->next = var;
 }
 
+static void delete_section_vars(config_section *pSection)
+{
+    config_var *curr_var;
+
+    if (pSection == NULL)
+        return;
+
+    curr_var = pSection->first_var;
+    while (curr_var != NULL)
+    {
+        config_var *next_var = curr_var->next;
+        if (curr_var->val_string != NULL)
+            free(curr_var->val_string);
+        if (curr_var->comment != NULL)
+            free(curr_var->comment);
+        free(curr_var);
+        curr_var = next_var;
+    }
+}
+
+static void delete_list(config_list *pConfigList)
+{
+    config_section *curr_section = *pConfigList;
+    while (curr_section != NULL)
+    {
+        config_section *next_section = curr_section->next;
+        /* delete all the variables in this section */
+        delete_section_vars(curr_section);
+        /* delete the section itself */
+        free(curr_section);
+        curr_section = next_section;
+    }
+
+    *pConfigList = NULL;
+}
+
+static config_section * section_deepcopy(config_section *orig_section, config_section *new_section)
+{
+    config_var *orig_var, *last_new_var;
+
+    /* Input validation */
+    if (orig_section == NULL)
+        return NULL;
+
+    /* create and copy section struct */
+    if (new_section == NULL)
+        new_section = (config_section *) malloc(sizeof(config_section));
+    if (new_section == NULL)
+        return NULL;
+    new_section->magic = SECTION_MAGIC;
+    memcpy(new_section->name, orig_section->name, 64);
+    new_section->name[63] = 0;
+    new_section->first_var = NULL;
+    new_section->next = NULL;
+
+    /* create and copy all section variables */
+    orig_var = orig_section->first_var;
+    last_new_var = NULL;
+    while (orig_var != NULL)
+    {
+        config_var *new_var = (config_var *) malloc(sizeof(config_var));
+        if (new_var == NULL)
+        {
+            delete_section_vars(new_section);
+            free(new_section);
+            return NULL;
+        }
+        memcpy(new_var->name, orig_var->name, 64);
+        new_var->name[63] = 0;
+        new_var->type = orig_var->type;
+        new_var->val_int = orig_var->val_int;
+        new_var->val_float = orig_var->val_float;
+        new_var->val_string = NULL;
+        new_var->comment = NULL;
+        new_var->next = NULL;
+        /* allocate memory and copy string values */
+        if (orig_var->val_string != NULL)
+        {
+            int len = strlen(orig_var->val_string);
+            new_var->val_string = (char *) malloc(len + 1);
+            if (new_var->val_string == NULL)
+            {
+                delete_section_vars(new_section);
+                free(new_section);
+                return NULL;
+            }
+            memcpy(new_var->val_string, orig_var->val_string, len + 1);
+        }
+        if (orig_var->comment != NULL)
+        {
+            int len = strlen(orig_var->comment);
+            new_var->comment = (char *) malloc(len + 1);
+            if (new_var->comment == NULL)
+            {
+                delete_section_vars(new_section);
+                free(new_section);
+                return NULL;
+            }
+            memcpy(new_var->comment, orig_var->comment, len + 1);
+        }
+        /* add the new variable to the new section */
+        if (last_new_var == NULL)
+            new_section->first_var = new_var;
+        else
+            last_new_var->next = new_var;
+        last_new_var = new_var;
+        /* advance variable pointer in original section variable list */
+        orig_var = orig_var->next;
+    }
+
+    return new_section;
+}
+
+static void copy_configlist_active_to_saved(void)
+{
+    config_section *curr_section = l_ConfigListActive;
+    config_section *last_section = NULL;
+
+    /* delete any pre-existing Saved config list */
+    delete_list(&l_ConfigListSaved);
+
+    /* duplicate all of the config sections in the Active list, adding them to the Saved list */
+    while (curr_section != NULL)
+    {
+        config_section *new_section = section_deepcopy(curr_section, NULL);
+        if (new_section == NULL) break;
+        if (last_section == NULL)
+            l_ConfigListSaved = new_section;
+        else
+            last_section->next = new_section;
+        last_section = new_section;
+        curr_section = curr_section->next;
+    }
+}
+
+static m64p_error write_configlist_file(void)
+{
+    config_section *curr_section;
+    const char *configpath;
+    char *filepath;
+    FILE *fPtr;
+
+    /* get the full pathname to the config file and try to open it */
+    configpath = ConfigGetUserConfigPath();
+    if (configpath == NULL)
+        return M64ERR_FILES;
+
+    filepath = (char *) malloc(strlen(configpath) + 32);
+    if (filepath == NULL)
+        return M64ERR_NO_MEMORY;
+
+    strcpy(filepath, configpath);
+    strcat(filepath, MUPEN64PLUS_CFG_NAME);
+    fPtr = fopen(filepath, "wb"); 
+    if (fPtr == NULL)
+    {
+        DebugMessage(M64MSG_ERROR, "Couldn't open configuration file '%s' for writing.", filepath);
+        free(filepath);
+        return M64ERR_FILES;
+    }
+    free(filepath);
+
+    /* write out header */
+    fprintf(fPtr, "# Mupen64Plus Configuration File\n");
+    fprintf(fPtr, "# This file is automatically read and written by the Mupen64Plus Core library\n");
+
+    /* write out all of the config parameters from the Saved list */
+    curr_section = l_ConfigListSaved;
+    while (curr_section != NULL)
+    {
+        config_var *curr_var = curr_section->first_var;
+        fprintf(fPtr, "\n[%s]\n\n", curr_section->name);
+        while (curr_var != NULL)
+        {
+            if (curr_var->comment != NULL && strlen(curr_var->comment) > 0)
+                fprintf(fPtr, "# %s\n", curr_var->comment);
+            if (curr_var->type == M64TYPE_INT)
+                fprintf(fPtr, "%s = %i\n", curr_var->name, curr_var->val_int);
+            else if (curr_var->type == M64TYPE_FLOAT)
+                fprintf(fPtr, "%s = %f\n", curr_var->name, curr_var->val_float);
+            else if (curr_var->type == M64TYPE_BOOL && curr_var->val_int)
+                fprintf(fPtr, "%s = True\n", curr_var->name);
+            else if (curr_var->type == M64TYPE_BOOL && !curr_var->val_int)
+                fprintf(fPtr, "%s = False\n", curr_var->name);
+            else if (curr_var->type == M64TYPE_STRING && curr_var->val_string != NULL)
+                fprintf(fPtr, "%s = \"%s\"\n", curr_var->name, curr_var->val_string);
+            curr_var = curr_var->next;
+        }
+        fprintf(fPtr, "\n");
+        curr_section = curr_section->next;
+    }
+
+    fclose(fPtr);
+    return M64ERR_SUCCESS;
+}
+
 /* ----------------------------------------------------------- */
 /* these functions are only to be used within the Core library */
 /* ----------------------------------------------------------- */
 
 m64p_error ConfigInit(const char *ConfigDirOverride, const char *DataDirOverride)
 {
+    m64p_error rval;
     const char *configpath = NULL;
     char *filepath;
     long filelen, pathlen;
@@ -175,16 +378,21 @@ m64p_error ConfigInit(const char *ConfigDirOverride, const char *DataDirOverride
         strcpy(l_DataDirOverride, DataDirOverride);
     }
 
-    /* get the full pathname to the config file and try to open it */
+    /* if a config directory was specified, make a copy of it */
     if (ConfigDirOverride != NULL)
-        configpath = ConfigDirOverride;
-    else
     {
-        configpath = ConfigGetUserConfigPath();
-        if (configpath == NULL)
-            return M64ERR_FILES;
+        l_ConfigDirOverride = (char *) malloc(strlen(ConfigDirOverride) + 1);
+        if (l_ConfigDirOverride == NULL)
+            return M64ERR_NO_MEMORY;
+        strcpy(l_ConfigDirOverride, ConfigDirOverride);
     }
 
+
+    /* get the full pathname to the config file and try to open it */
+    configpath = ConfigGetUserConfigPath();
+    if (configpath == NULL)
+        return M64ERR_FILES;
+
     filepath = (char *) malloc(strlen(configpath) + 32);
     if (filepath == NULL)
         return M64ERR_NO_MEMORY;
@@ -200,7 +408,7 @@ m64p_error ConfigInit(const char *ConfigDirOverride, const char *DataDirOverride
     fPtr = fopen(filepath, "rb");
     if (fPtr == NULL)
     {
-        DebugMessage(M64MSG_WARNING, "Couldn't open configuration file '%s'.  Using defaults.", filepath);
+        DebugMessage(M64MSG_INFO, "Couldn't open configuration file '%s'.  Using defaults.", filepath);
         free(filepath);
         l_SaveConfigOnExit = 1; /* auto-save the config file so that the defaults will be saved to disk */
         return M64ERR_SUCCESS;
@@ -258,25 +466,15 @@ m64p_error ConfigInit(const char *ConfigDirOverride, const char *DataDirOverride
         /* handle section definition line */
         if (strlen(line) > 2 && line[0] == '[' && line[strlen(line)-1] == ']')
         {
-            config_section *last_section = current_section;
             line++;
             line[strlen(line)-1] = 0;
-            current_section = (config_section *) malloc(sizeof(config_section));
-            if (current_section == NULL)
+            rval = ConfigOpenSection(line, (m64p_handle *) &current_section);
+            if (rval != M64ERR_SUCCESS)
             {
                 free(configtext);
-                return M64ERR_NO_MEMORY;
+                return rval;
             }
-            current_section->magic = SECTION_MAGIC;
-            strncpy(current_section->name, line, 63);
-            current_section->name[63] = 0;
-            current_section->first_var = NULL;
-            current_section->next = NULL;
             lastcomment = NULL;
-            if (last_section == NULL)
-                l_SectionHead = current_section;
-            else
-                last_section->next = current_section;
             line = nextline;
             continue;
         }
@@ -324,14 +522,17 @@ m64p_error ConfigInit(const char *ConfigDirOverride, const char *DataDirOverride
         line = nextline;
     }
 
+    /* release memory used for config file text */
     free(configtext);
+
+    /* duplicate the entire config data list, to store a copy of the list which represents the state of the file on disk */
+    copy_configlist_active_to_saved();
+
     return M64ERR_SUCCESS;
 }
 
 m64p_error ConfigShutdown(void)
 {
-    config_section *curr_section;
-
     /* first, save the file if necessary */
     if (l_SaveConfigOnExit)
         ConfigSaveFile();
@@ -347,30 +548,16 @@ m64p_error ConfigShutdown(void)
         free(l_DataDirOverride);
         l_DataDirOverride = NULL;
     }
-
-    /* free all of the malloc'd blocks */
-    curr_section = l_SectionHead;
-    while (curr_section != NULL)
+    if (l_ConfigDirOverride != NULL)
     {
-        config_section *next_section = curr_section->next;
-        /* delete all the variables in this section */
-        config_var *curr_var = curr_section->first_var;
-        while (curr_var != NULL)
-        {
-            config_var *next_var = curr_var->next;
-            if (curr_var->val_string != NULL)
-                free(curr_var->val_string);
-            if (curr_var->comment != NULL)
-                free(curr_var->comment);
-            free(curr_var);
-            curr_var = next_var;
-        }
-        /* delete the section itself */
-        free(curr_section);
-        curr_section = next_section;
+        free(l_ConfigDirOverride);
+        l_ConfigDirOverride = NULL;
     }
 
-    l_SectionHead = NULL;
+    /* free all of the memory in the 2 lists */
+    delete_list(&l_ConfigListActive);
+    delete_list(&l_ConfigListSaved);
+
     return M64ERR_SUCCESS;
 }
 
@@ -388,7 +575,7 @@ EXPORT m64p_error CALL ConfigListSections(void *context, void (*SectionListCallb
         return M64ERR_INPUT_ASSERT;
 
     /* just walk through the section list, making a callback for each section name */
-    curr_section = l_SectionHead;
+    curr_section = l_ConfigListActive;
     while (curr_section != NULL)
     {
         (*SectionListCallback)(context, curr_section->name);
@@ -408,7 +595,7 @@ EXPORT m64p_error CALL ConfigOpenSection(const char *SectionName, m64p_handle *C
         return M64ERR_INPUT_ASSERT;
 
     /* walk through the section list, looking for a case-insensitive name match */
-    curr_section = l_SectionHead;
+    curr_section = l_ConfigListActive;
     while (curr_section != NULL)
     {
         if (osal_insensitive_strcmp(SectionName, curr_section->name) == 0)
@@ -429,14 +616,18 @@ EXPORT m64p_error CALL ConfigOpenSection(const char *SectionName, m64p_handle *C
     new_section->first_var = NULL;
     new_section->next = NULL;
 
-    /* add section to the end of the list */
-    if (l_SectionHead == NULL)
-        l_SectionHead = new_section;
+    /* add section to list in alphabetical order */
+    if (l_ConfigListActive == NULL || osal_insensitive_strcmp(SectionName, l_ConfigListActive->name) < 0)
+    {
+        new_section->next = l_ConfigListActive;
+        l_ConfigListActive = new_section;
+    }
     else
     {
-        curr_section = l_SectionHead;
-        while (curr_section->next != NULL)
+        curr_section = l_ConfigListActive;
+        while (curr_section->next != NULL && osal_insensitive_strcmp(SectionName, curr_section->next->name) >= 0)
             curr_section = curr_section->next;
+        new_section->next = curr_section->next;
         curr_section->next = new_section;
     }
 
@@ -469,68 +660,123 @@ EXPORT m64p_error CALL ConfigListParameters(m64p_handle ConfigSectionHandle, voi
   return M64ERR_SUCCESS;
 }
 
-EXPORT m64p_error CALL ConfigSaveFile(void)
+EXPORT int CALL ConfigHasUnsavedChanges(const char *SectionName)
 {
-    config_section *curr_section;
-    const char *configpath;
-    char *filepath;
-    FILE *fPtr;
+    config_section *input_section, *curr_section;
+    config_var *active_var, *saved_var;
 
+    /* check input conditions */
     if (!l_ConfigInit)
-        return M64ERR_NOT_INIT;
-
-    /* get the full pathname to the config file and try to open it */
-    configpath = ConfigGetUserConfigPath();
-    if (configpath == NULL)
-        return M64ERR_FILES;
-
-    filepath = (char *) malloc(strlen(configpath) + 32);
-    if (filepath == NULL)
-        return M64ERR_NO_MEMORY;
-
-    strcpy(filepath, configpath);
-    strcat(filepath, MUPEN64PLUS_CFG_NAME);
-    fPtr = fopen(filepath, "wb"); 
-    if (fPtr == NULL)
     {
-        DebugMessage(M64MSG_ERROR, "Couldn't open configuration file '%s' for writing.", filepath);
-        free(filepath);
-        return M64ERR_FILES;
+        DebugMessage(M64MSG_ERROR, "ConfigHasUnsavedChanges(): Core config not initialized!");
+        return 0;
     }
-    free(filepath);
 
-    /* write out header */
-    fprintf(fPtr, "# Mupen64Plus Configuration File\n");
-    fprintf(fPtr, "# This file is automatically read and written by the Mupen64Plus Core library\n");
+    /* if SectionName is NULL or blank, then check all sections */
+    if (SectionName == NULL || strlen(SectionName) < 1)
+    {
+        int iNumActiveSections = 0, iNumSavedSections = 0;
+        /* first, search through all sections in Active list.  Recursively call ourself and return 1 if changed */
+        curr_section = l_ConfigListActive;
+        while (curr_section != NULL)
+        {
+            if (ConfigHasUnsavedChanges(curr_section->name))
+                return 1;
+            curr_section = curr_section->next;
+            iNumActiveSections++;
+        }
+        /* Next, count the number of Saved sections and see if the count matches */
+        curr_section = l_ConfigListSaved;
+        while (curr_section != NULL)
+        {
+            curr_section = curr_section->next;
+            iNumSavedSections++;
+        }
+        if (iNumActiveSections == iNumSavedSections)
+            return 0;  /* no changes */
+        else
+            return 1;
+    }
 
-    /* write out all of the config parameters */
-    curr_section = l_SectionHead;
+    /* walk through the Active section list, looking for a case-insensitive name match with input string */
+    input_section = l_ConfigListActive;
+    while (input_section != NULL)
+    {
+        if (osal_insensitive_strcmp(SectionName, input_section->name) == 0)
+            break;
+        input_section = input_section->next;
+    }
+    if (input_section == NULL)
+    {
+        DebugMessage(M64MSG_ERROR, "ConfigHasUnsavedChanges(): section name '%s' not found!", SectionName);
+        return 0;
+    }
+
+    /* walk through the Saved section list, looking for a case-insensitive name match */
+    curr_section = l_ConfigListSaved;
     while (curr_section != NULL)
     {
-        config_var *curr_var = curr_section->first_var;
-        fprintf(fPtr, "\n[%s]\n\n", curr_section->name);
-        while (curr_var != NULL)
-        {
-            if (curr_var->comment != NULL && strlen(curr_var->comment) > 0)
-                fprintf(fPtr, "# %s\n", curr_var->comment);
-            if (curr_var->type == M64TYPE_INT)
-                fprintf(fPtr, "%s = %i\n", curr_var->name, curr_var->val_int);
-            else if (curr_var->type == M64TYPE_FLOAT)
-                fprintf(fPtr, "%s = %f\n", curr_var->name, curr_var->val_float);
-            else if (curr_var->type == M64TYPE_BOOL && curr_var->val_int)
-                fprintf(fPtr, "%s = True\n", curr_var->name);
-            else if (curr_var->type == M64TYPE_BOOL && !curr_var->val_int)
-                fprintf(fPtr, "%s = False\n", curr_var->name);
-            else if (curr_var->type == M64TYPE_STRING && curr_var->val_string != NULL)
-                fprintf(fPtr, "%s = \"%s\"\n", curr_var->name, curr_var->val_string);
-            curr_var = curr_var->next;
-        }
-        fprintf(fPtr, "\n");
+        if (osal_insensitive_strcmp(input_section->name, curr_section->name) == 0)
+            break;
         curr_section = curr_section->next;
     }
+    /* if this section isn't present in saved list, then it has been newly created */
+    if (curr_section == NULL)
+        return 1;
 
-    fclose(fPtr);
-    return M64ERR_SUCCESS;
+    /* compare all of the variables in the two sections. They are expected to be in the same order */
+    active_var = input_section->first_var;
+    saved_var = curr_section->first_var;
+    while (active_var != NULL && saved_var != NULL)
+    {
+        if (strncmp(active_var->name, saved_var->name, 64) != 0)
+            return 1;
+        if (active_var->type != saved_var->type)
+            return 1;
+        switch(active_var->type)
+        {
+            case M64TYPE_INT:
+                if (active_var->val_int != saved_var->val_int)
+                    return 1;
+                break;
+            case M64TYPE_FLOAT:
+                if (active_var->val_float != saved_var->val_float)
+                    return 1;
+                break;
+            case M64TYPE_BOOL:
+                if ((active_var->val_int != 0) != (saved_var->val_int != 0))
+                    return 1;
+                break;
+            case M64TYPE_STRING:
+                if (active_var->val_string == NULL)
+                {
+                    DebugMessage(M64MSG_ERROR, "ConfigHasUnsavedChanges(): Variable '%s' NULL Active string pointer!", active_var->name);
+                    return 1;
+                }
+                if (saved_var->val_string == NULL)
+                {
+                    DebugMessage(M64MSG_ERROR, "ConfigHasUnsavedChanges(): Variable '%s' NULL Saved string pointer!", active_var->name);
+                    return 1;
+                }
+                if (strcmp(active_var->val_string, saved_var->val_string) != 0)
+                    return 1;
+                break;
+            default:
+                DebugMessage(M64MSG_ERROR, "ConfigHasUnsavedChanges(): Invalid variable '%s' type %i!", active_var->name, active_var->type);
+                return 1;
+        }
+        if (active_var->comment != NULL && saved_var->comment != NULL && strcmp(active_var->comment, saved_var->comment) != 0)
+            return 1;
+        active_var = active_var->next;
+        saved_var = saved_var->next;
+    }
+
+    /* any extra new variables on the end, or deleted variables? */
+    if (active_var != NULL || saved_var != NULL)
+        return 1;
+
+    /* exactly the same */
+    return 0;
 }
 
 /* ------------------------------------------------------- */
@@ -544,14 +790,14 @@ EXPORT m64p_error CALL ConfigDeleteSection(const char *SectionName)
 
     if (!l_ConfigInit)
         return M64ERR_NOT_INIT;
-    if (l_SectionHead == NULL)
+    if (l_ConfigListActive == NULL)
         return M64ERR_INPUT_NOT_FOUND;
 
     /* find the named section and pull it out of the list */
-    curr_section = l_SectionHead;
-    if (osal_insensitive_strcmp(l_SectionHead->name, SectionName) == 0)
+    curr_section = l_ConfigListActive;
+    if (osal_insensitive_strcmp(l_ConfigListActive->name, SectionName) == 0)
     {
-        l_SectionHead = l_SectionHead->next;
+        l_ConfigListActive = l_ConfigListActive->next;
     }
     else
     {
@@ -588,6 +834,132 @@ EXPORT m64p_error CALL ConfigDeleteSection(const char *SectionName)
     return M64ERR_SUCCESS;
 }
 
+EXPORT m64p_error CALL ConfigSaveFile(void)
+{
+    if (!l_ConfigInit)
+        return M64ERR_NOT_INIT;
+
+    /* copy the active config list to the saved config list */
+    copy_configlist_active_to_saved();
+
+    /* write the saved config list out to a file */
+    return (write_configlist_file());
+}
+
+EXPORT m64p_error CALL ConfigSaveSection(const char *SectionName)
+{
+    config_section *curr_section, *new_section;
+
+    if (!l_ConfigInit)
+        return M64ERR_NOT_INIT;
+    if (SectionName == NULL || strlen(SectionName) < 1)
+        return M64ERR_INPUT_ASSERT;
+
+    /* walk through the Active section list, looking for a case-insensitive name match */
+    curr_section = l_ConfigListActive;
+    while (curr_section != NULL)
+    {
+        if (osal_insensitive_strcmp(SectionName, curr_section->name) == 0)
+            break;
+        curr_section = curr_section->next;
+    }
+    if (curr_section == NULL)
+        return M64ERR_INPUT_NOT_FOUND;
+
+    /* duplicate this section */
+    new_section = section_deepcopy(curr_section, NULL);
+    if (new_section == NULL)
+        return M64ERR_NO_MEMORY;
+
+    /* update config section that's in the Saved list with the new one */
+    if (l_ConfigListSaved == NULL || osal_insensitive_strcmp(SectionName, l_ConfigListSaved->name) < 0)
+    {
+        /* the saved section is new and goes at the beginning of the list */
+        new_section->next = l_ConfigListSaved;
+        l_ConfigListSaved = new_section;
+    }
+    else if (osal_insensitive_strcmp(SectionName, l_ConfigListSaved->name) == 0)
+    {
+        /* the saved section replaces the first section in the list */
+        new_section->next = l_ConfigListSaved->next;
+        delete_section_vars(l_ConfigListSaved);
+        free(l_ConfigListSaved);
+        l_ConfigListSaved = new_section;
+    }
+    else
+    {
+        curr_section = l_ConfigListSaved;
+        while (curr_section->next != NULL && osal_insensitive_strcmp(SectionName, curr_section->next->name) > 0)
+            curr_section = curr_section->next;
+        if (curr_section->next == NULL || osal_insensitive_strcmp(SectionName, curr_section->next->name) < 0)
+        {
+            /* the saved section is new and goes after the curr_section */
+            new_section->next = curr_section->next;
+            curr_section->next = new_section;
+        }
+        else
+        {
+            /* the saved section replaces curr_section->next */
+            config_section *old_section = curr_section->next;
+            new_section->next = old_section->next;
+            delete_section_vars(old_section);
+            free(old_section);
+            curr_section->next = new_section;
+        }
+    }
+
+    /* write the saved config list out to a file */
+    return (write_configlist_file());
+}
+
+EXPORT m64p_error CALL ConfigRevertChanges(const char *SectionName)
+{
+    config_section *input_section, *curr_section, *new_section, *temp_next_ptr;
+
+    /* check input conditions */
+    if (!l_ConfigInit)
+        return M64ERR_NOT_INIT;
+    if (SectionName == NULL)
+        return M64ERR_INPUT_ASSERT;
+
+    /* walk through the Active section list, looking for a case-insensitive name match with input string */
+    input_section = l_ConfigListActive;
+    while (input_section != NULL)
+    {
+        if (osal_insensitive_strcmp(SectionName, input_section->name) == 0)
+            break;
+        input_section = input_section->next;
+    }
+    if (input_section == NULL)
+        return M64ERR_INPUT_NOT_FOUND;
+
+    /* walk through the Saved section list, looking for a case-insensitive name match */
+    curr_section = l_ConfigListSaved;
+    while (curr_section != NULL)
+    {
+        if (osal_insensitive_strcmp(SectionName, curr_section->name) == 0)
+            break;
+        curr_section = curr_section->next;
+    }
+    /* if this section isn't present in saved list, then it has been newly created */
+    if (curr_section == NULL)
+        return M64ERR_INPUT_NOT_FOUND;
+
+    /* we need to save the "next" pointer in the active section, because this will get blown away by the deepcopy */
+    temp_next_ptr = input_section->next;
+    /* delete the variables from the Active section */
+    delete_section_vars(input_section);
+    /* copy all of the section data from the Saved section to the Active one */
+    new_section = section_deepcopy(curr_section, input_section);
+    if (new_section == NULL)
+        return M64ERR_NO_MEMORY;  /* it's very bad if this happens, because original data from Active section has been deleted */
+    /* new_section should be == to input_section.  now put the "next" pointer back */
+    input_section->next = temp_next_ptr;
+
+    /* should be good to go */
+    return M64ERR_SUCCESS;
+}
+
 
 /* ------------------------------------------------------- */
 /* Generic Get/Set functions, exported outside of the Core */
@@ -1150,7 +1522,10 @@ EXPORT const char * CALL ConfigGetSharedDataFilepath(const char *filename)
 
 EXPORT const char * CALL ConfigGetUserConfigPath(void)
 {
-  return osal_get_user_configpath();
+    if (l_ConfigDirOverride != NULL)
+        return l_ConfigDirOverride;
+    else
+        return osal_get_user_configpath();
 }
 
 EXPORT const char * CALL ConfigGetUserDataPath(void)
