@@ -23,13 +23,21 @@
 #include <stdlib.h>
 #include <SDL.h>
 
+#define M64P_CORE_PROTOTYPES 1
 #include "main.h"
 #include "eventloop.h"
 #include "util.h"
+#include "api/callbacks.h"
 #include "api/config.h"
+#include "api/m64p_config.h"
 #include "plugin/plugin.h"
 #include "r4300/interupt.h"
 
+/* version number for CoreEvents config section */
+#define CONFIG_PARAM_VERSION 1.00
+
+static m64p_handle l_CoreEventsConfig = NULL;
+
 /*********************************************************************************************************
 * static variables and definitions for eventloop.c
 */
@@ -65,20 +73,20 @@ typedef enum {joyFullscreen,
               joyGameshark
 } eJoyCommand;
 
-const char *JoyCmdName[] = { "Joy Mapping Fullscreen",
-                             "Joy Mapping Stop",
-                             "Joy Mapping Pause",
-                             "Joy Mapping Save State",
-                             "Joy Mapping Load State",
-                             "Joy Mapping Increment Slot",
-                             "Joy Mapping Screenshot",
-                             "Joy Mapping Mute",
-                             "Joy Mapping Increase Volume",
-                             "Joy Mapping Decrease Volume",
-                             "Joy Mapping Fast Forward",
-                             "Joy Mapping Gameshark"};
+static const char *JoyCmdName[] = { "Joy Mapping Fullscreen",
+                                    "Joy Mapping Stop",
+                                    "Joy Mapping Pause",
+                                    "Joy Mapping Save State",
+                                    "Joy Mapping Load State",
+                                    "Joy Mapping Increment Slot",
+                                    "Joy Mapping Screenshot",
+                                    "Joy Mapping Mute",
+                                    "Joy Mapping Increase Volume",
+                                    "Joy Mapping Decrease Volume",
+                                    "Joy Mapping Fast Forward",
+                                    "Joy Mapping Gameshark"};
 
-const int NumJoyCommands = sizeof(JoyCmdName) / sizeof(const char *);
+static const int NumJoyCommands = sizeof(JoyCmdName) / sizeof(const char *);
 
 static int JoyCmdActive[16];  /* if extra joystick commands are added above, make sure there is enough room in this array */
 
@@ -98,7 +106,7 @@ static int KbdGamesharkPressed = 0;
  */
 static int MatchJoyCommand(const SDL_Event *event, eJoyCommand cmd)
 {
-    const char *event_str = ConfigGetParamString(g_CoreConfig, JoyCmdName[cmd]);
+    const char *event_str = ConfigGetParamString(l_CoreEventsConfig, JoyCmdName[cmd]);
     int dev_number, input_number, input_value;
     char axis_direction;
 
@@ -289,7 +297,7 @@ void event_initialize(void)
     /* activate any joysticks which are referenced in the joystick event command strings */
     for (i = 0; i < NumJoyCommands; i++)
     {
-        event_str = ConfigGetParamString(g_CoreConfig, JoyCmdName[i]);
+        event_str = ConfigGetParamString(l_CoreEventsConfig, JoyCmdName[i]);
         if (event_str != NULL && strlen(event_str) >= 4 && event_str[0] == 'J' && event_str[1] >= '0' && event_str[1] <= '9')
         {
             int device = event_str[1] - '0';
@@ -305,38 +313,76 @@ void event_initialize(void)
     SDL_SetEventFilter(event_sdl_filter);
 }
 
-void event_set_core_defaults(void)
+int event_set_core_defaults(void)
 {
+    float fConfigParamsVersion;
+    int bSaveConfig = 0;
+
+    if (ConfigOpenSection("CoreEvents", &l_CoreEventsConfig) != M64ERR_SUCCESS || l_CoreEventsConfig == NULL)
+    {
+        DebugMessage(M64MSG_ERROR, "Failed to open CoreEvents config section.");
+        return 0; /* fail */
+    }
+
+    if (ConfigGetParameter(l_CoreEventsConfig, "Version", M64TYPE_FLOAT, &fConfigParamsVersion, sizeof(float)) != M64ERR_SUCCESS)
+    {
+        DebugMessage(M64MSG_WARNING, "No version number in 'CoreEvents' config section. Setting defaults.");
+        ConfigDeleteSection("CoreEvents");
+        ConfigOpenSection("CoreEvents", &l_CoreEventsConfig);
+        bSaveConfig = 1;
+    }
+    else if (((int) fConfigParamsVersion) != ((int) CONFIG_PARAM_VERSION))
+    {
+        DebugMessage(M64MSG_WARNING, "Incompatible version %.2f in 'CoreEvents' config section: current is %.2f. Setting defaults.", fConfigParamsVersion, (float) CONFIG_PARAM_VERSION);
+        ConfigDeleteSection("CoreEvents");
+        ConfigOpenSection("CoreEvents", &l_CoreEventsConfig);
+        bSaveConfig = 1;
+    }
+    else if ((CONFIG_PARAM_VERSION - fConfigParamsVersion) >= 0.0001f)
+    {
+        /* handle upgrades */
+        float fVersion = CONFIG_PARAM_VERSION;
+        ConfigSetParameter(l_CoreEventsConfig, "Version", M64TYPE_FLOAT, &fVersion);
+        DebugMessage(M64MSG_INFO, "Updating parameter set version in 'CoreEvents' config section to %.2f", fVersion);
+        bSaveConfig = 1;
+    }
+
+    ConfigSetDefaultFloat(l_CoreEventsConfig, "Version", CONFIG_PARAM_VERSION,  "Mupen64Plus CoreEvents config parameter set version number.  Please don't change");
     /* Keyboard presses mapped to core functions */
-    ConfigSetDefaultInt(g_CoreConfig, kbdStop, SDLK_ESCAPE,          "SDL keysym for stopping the emulator");
-    ConfigSetDefaultInt(g_CoreConfig, kbdFullscreen, SDLK_LAST,      "SDL keysym for switching between fullscreen/windowed modes");
-    ConfigSetDefaultInt(g_CoreConfig, kbdSave, SDLK_F5,              "SDL keysym for saving the emulator state");
-    ConfigSetDefaultInt(g_CoreConfig, kbdLoad, SDLK_F7,              "SDL keysym for loading the emulator state");
-    ConfigSetDefaultInt(g_CoreConfig, kbdIncrement, 0,               "SDL keysym for advancing the save state slot");
-    ConfigSetDefaultInt(g_CoreConfig, kbdReset, SDLK_F9,             "SDL keysym for resetting the emulator");
-    ConfigSetDefaultInt(g_CoreConfig, kbdSpeeddown, SDLK_F10,        "SDL keysym for slowing down the emulator");
-    ConfigSetDefaultInt(g_CoreConfig, kbdSpeedup, SDLK_F11,          "SDL keysym for speeding up the emulator");
-    ConfigSetDefaultInt(g_CoreConfig, kbdScreenshot, SDLK_F12,       "SDL keysym for taking a screenshot");
-    ConfigSetDefaultInt(g_CoreConfig, kbdPause, SDLK_p,              "SDL keysym for pausing the emulator");
-    ConfigSetDefaultInt(g_CoreConfig, kbdMute, SDLK_m,               "SDL keysym for muting/unmuting the sound");
-    ConfigSetDefaultInt(g_CoreConfig, kbdIncrease, SDLK_RIGHTBRACKET,"SDL keysym for increasing the volume");
-    ConfigSetDefaultInt(g_CoreConfig, kbdDecrease, SDLK_LEFTBRACKET, "SDL keysym for decreasing the volume");
-    ConfigSetDefaultInt(g_CoreConfig, kbdForward, SDLK_f,            "SDL keysym for temporarily going really fast");
-    ConfigSetDefaultInt(g_CoreConfig, kbdAdvance, SDLK_SLASH,        "SDL keysym for advancing by one frame when paused");
-    ConfigSetDefaultInt(g_CoreConfig, kbdGameshark, SDLK_g,          "SDL keysym for pressing the game shark button");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdStop, SDLK_ESCAPE,          "SDL keysym for stopping the emulator");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdFullscreen, SDLK_LAST,      "SDL keysym for switching between fullscreen/windowed modes");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdSave, SDLK_F5,              "SDL keysym for saving the emulator state");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdLoad, SDLK_F7,              "SDL keysym for loading the emulator state");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdIncrement, 0,               "SDL keysym for advancing the save state slot");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdReset, SDLK_F9,             "SDL keysym for resetting the emulator");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdSpeeddown, SDLK_F10,        "SDL keysym for slowing down the emulator");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdSpeedup, SDLK_F11,          "SDL keysym for speeding up the emulator");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdScreenshot, SDLK_F12,       "SDL keysym for taking a screenshot");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdPause, SDLK_p,              "SDL keysym for pausing the emulator");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdMute, SDLK_m,               "SDL keysym for muting/unmuting the sound");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdIncrease, SDLK_RIGHTBRACKET,"SDL keysym for increasing the volume");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdDecrease, SDLK_LEFTBRACKET, "SDL keysym for decreasing the volume");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdForward, SDLK_f,            "SDL keysym for temporarily going really fast");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdAdvance, SDLK_SLASH,        "SDL keysym for advancing by one frame when paused");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdGameshark, SDLK_g,          "SDL keysym for pressing the game shark button");
     /* Joystick events mapped to core functions */
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyStop], "",       "Joystick event string for stopping the emulator");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyFullscreen], "", "Joystick event string for switching between fullscreen/windowed modes");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joySave], "",       "Joystick event string for saving the emulator state");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyLoad], "",       "Joystick event string for loading the emulator state");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyIncrement], "",  "Joystick event string for advancing the save state slot");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyScreenshot], "", "Joystick event string for taking a screenshot");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyPause], "",      "Joystick event string for pausing the emulator");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyMute], "",       "Joystick event string for muting/unmuting the sound");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyIncrease], "",   "Joystick event string for increasing the volume");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyDecrease], "",   "Joystick event string for decreasing the volume");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyForward], "",    "Joystick event string for fast-forward");
-    ConfigSetDefaultString(g_CoreConfig, JoyCmdName[joyGameshark], "",  "Joystick event string for pressing the game shark button");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyStop], "",       "Joystick event string for stopping the emulator");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyFullscreen], "", "Joystick event string for switching between fullscreen/windowed modes");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joySave], "",       "Joystick event string for saving the emulator state");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyLoad], "",       "Joystick event string for loading the emulator state");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyIncrement], "",  "Joystick event string for advancing the save state slot");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyScreenshot], "", "Joystick event string for taking a screenshot");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyPause], "",      "Joystick event string for pausing the emulator");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyMute], "",       "Joystick event string for muting/unmuting the sound");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyIncrease], "",   "Joystick event string for increasing the volume");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyDecrease], "",   "Joystick event string for decreasing the volume");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyForward], "",    "Joystick event string for fast-forward");
+    ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyGameshark], "",  "Joystick event string for pressing the game shark button");
+
+    if (bSaveConfig)
+        ConfigSaveSection("CoreEvents");
+
+    return 1;
 }
 
 /*********************************************************************************************************
@@ -355,53 +401,53 @@ void event_sdl_keydown(int keysym, int keymod)
         main_state_set_slot(keysym - SDLK_0);
     }
     /* check all of the configurable commands */
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdStop))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdStop))
         main_stop();
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdFullscreen))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdFullscreen))
         changeWindow();
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdSave))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdSave))
         main_state_save(0, NULL); /* save in mupen64plus format using current slot */
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdLoad))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdLoad))
         main_state_load(NULL); /* load using current slot */
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdIncrement))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdIncrement))
         main_state_inc_slot();
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdReset))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdReset))
     {
         add_interupt_event(HW2_INT, 0);  /* Hardware 2 Interrupt immediately */
         add_interupt_event(NMI_INT, 50000000);  /* Non maskable Interrupt after 1/2 second */
     }
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdSpeeddown))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdSpeeddown))
         main_speeddown(5);
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdSpeedup))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdSpeedup))
         main_speedup(5);
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdScreenshot))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdScreenshot))
         main_take_next_screenshot();    /* screenshot will be taken at the end of frame rendering */
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdPause))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdPause))
         main_toggle_pause();
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdMute))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdMute))
     {
         volumeMute();
         main_draw_volume_osd();
     }
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdIncrease))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdIncrease))
     {
         volumeUp();
         main_draw_volume_osd();
     }
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdDecrease))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdDecrease))
     {
         volumeDown();
         main_draw_volume_osd();
     }
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdForward))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdForward))
     {
         main_set_fastforward(1);
     }
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdAdvance))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdAdvance))
     {
         main_advance_one();
     }
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdGameshark))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdGameshark))
     {
         KbdGamesharkPressed = 1;
     }
@@ -415,15 +461,15 @@ void event_sdl_keydown(int keysym, int keymod)
 
 void event_sdl_keyup(int keysym, int keymod)
 {
-    if (keysym == ConfigGetParamInt(g_CoreConfig, kbdStop))
+    if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdStop))
     {
         return;
     }
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdForward))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdForward))
     {
         main_set_fastforward(0);
     }
-    else if (keysym == ConfigGetParamInt(g_CoreConfig, kbdGameshark))
+    else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdGameshark))
     {
         KbdGamesharkPressed = 0;
     }
