@@ -32,9 +32,11 @@
 
 #include <SDL.h>
 
+#define M64P_CORE_PROTOTYPES 1
 #include "api/m64p_types.h"
 #include "api/callbacks.h"
 #include "api/config.h"
+#include "api/m64p_config.h"
 #include "api/debugger.h"
 #include "api/vidext.h"
 
@@ -60,6 +62,9 @@
 #include "lirc.h"
 #endif //WITH_LIRC
 
+/* version number for Core config section */
+#define CONFIG_PARAM_VERSION 1.01
+
 /** globals **/
 m64p_handle g_CoreConfig = NULL;
 
@@ -67,34 +72,58 @@ m64p_frame_callback g_FrameCallback = NULL;
 
 int         g_MemHasBeenBSwapped = 0;   // store byte-swapped flag so we don't swap twice when re-playing game
 int         g_EmulatorRunning = 0;      // need separate boolean to tell if emulator is running, since --nogui doesn't use a thread
-int         g_TakeScreenshot = 0;       // Tell OSD Rendering callback to take a screenshot just before drawing the OSD
 
 /** static (local) variables **/
 static int   l_CurrentFrame = 0;         // frame counter
+static int   l_TakeScreenshot = 0;       // Tell OSD Rendering callback to take a screenshot just before drawing the OSD
 static int   l_SpeedFactor = 100;        // percentage of nominal game speed at which emulator is running
 static int   l_FrameAdvance = 0;         // variable to check if we pause on next frame
+static int   l_MainSpeedLimit = 1;       // insert delay during vi_interrupt to keep speed at real-time
 
 static osd_message_t *l_msgVol = NULL;
 static osd_message_t *l_msgFF = NULL;
 static osd_message_t *l_msgPause = NULL;
 
 /*********************************************************************************************************
-* helper functions
+* static functions
 */
-char *get_savespath()
+
+static const char *get_savepathdefault(const char *configpath)
 {
     static char path[1024];
 
-    snprintf(path, 1024, "%ssave%c", ConfigGetUserDataPath(), OSAL_DIR_SEPARATOR);
-    path[1023] = 0;
+    if (!configpath || (strlen(configpath) == 0)) {
+        snprintf(path, 1024, "%ssave%c", ConfigGetUserDataPath(), OSAL_DIR_SEPARATOR);
+        path[1023] = 0;
+    } else {
+        snprintf(path, 1024, "%s%c", configpath, OSAL_DIR_SEPARATOR);
+        path[1023] = 0;
+    }
 
-    /* make sure the directory exists */
-    if (osal_mkdirp(path, 0700) != 0)
-        return NULL;
+    /* create directory if it doesn't exist */
+    osal_mkdirp(path, 0700);
 
     return path;
 }
 
+
+/*********************************************************************************************************
+* helper functions
+*/
+
+
+const char *get_savestatepath(void)
+{
+    /* try to get the SaveStatePath string variable in the Core configuration section */
+    return get_savepathdefault(ConfigGetParamString(g_CoreConfig, "SaveStatePath"));
+}
+
+const char *get_savesrampath(void)
+{
+    /* try to get the SaveSRAMPath string variable in the Core configuration section */
+    return get_savepathdefault(ConfigGetParamString(g_CoreConfig, "SaveSRAMPath"));
+}
+
 void main_message(m64p_msg_level level, unsigned int corner, const char *format, ...)
 {
     va_list ap;
@@ -106,9 +135,9 @@ void main_message(m64p_msg_level level, unsigned int corner, const char *format,
 
     /* send message to on-screen-display if enabled */
     if (ConfigGetParamBool(g_CoreConfig, "OnScreenDisplay"))
-        osd_new_message((enum osd_corner) corner, buffer);
+        osd_new_message((enum osd_corner) corner, "%s", buffer);
     /* send message to front-end */
-    DebugMessage(level, buffer);
+    DebugMessage(level, "%s", buffer);
 }
 
 
@@ -152,9 +181,36 @@ static int GetVILimit(void)
 * global functions, for adjusting the core emulator behavior
 */
 
-void main_set_core_defaults(void)
+int main_set_core_defaults(void)
 {
+    float fConfigParamsVersion;
+    int bSaveConfig = 0, bUpgrade = 0;
+
+    if (ConfigGetParameter(g_CoreConfig, "Version", M64TYPE_FLOAT, &fConfigParamsVersion, sizeof(float)) != M64ERR_SUCCESS)
+    {
+        DebugMessage(M64MSG_WARNING, "No version number in 'Core' config section. Setting defaults.");
+        ConfigDeleteSection("Core");
+        ConfigOpenSection("Core", &g_CoreConfig);
+        bSaveConfig = 1;
+    }
+    else if (((int) fConfigParamsVersion) != ((int) CONFIG_PARAM_VERSION))
+    {
+        DebugMessage(M64MSG_WARNING, "Incompatible version %.2f in 'Core' config section: current is %.2f. Setting defaults.", fConfigParamsVersion, (float) CONFIG_PARAM_VERSION);
+        ConfigDeleteSection("Core");
+        ConfigOpenSection("Core", &g_CoreConfig);
+        bSaveConfig = 1;
+    }
+    else if ((CONFIG_PARAM_VERSION - fConfigParamsVersion) >= 0.0001f)
+    {
+        float fVersion = (float) CONFIG_PARAM_VERSION;
+        ConfigSetParameter(g_CoreConfig, "Version", M64TYPE_FLOAT, &fVersion);
+        DebugMessage(M64MSG_INFO, "Updating parameter set version in 'Core' config section to %.2f", fVersion);
+        bUpgrade = 1;
+        bSaveConfig = 1;
+    }
+
     /* parameters controlling the operation of the core */
+    ConfigSetDefaultFloat(g_CoreConfig, "Version", (float) CONFIG_PARAM_VERSION,  "Mupen64Plus Core config parameter set version number.  Please don't change");
     ConfigSetDefaultBool(g_CoreConfig, "OnScreenDisplay", 1, "Draw on-screen display if True, otherwise don't draw OSD");
 #if defined(DYNAREC)
     ConfigSetDefaultInt(g_CoreConfig, "R4300Emulator", 2, "Use Pure Interpreter if 0, Cached Interpreter if 1, or Dynamic Recompiler if 2 or more");
@@ -167,11 +223,26 @@ void main_set_core_defaults(void)
     ConfigSetDefaultBool(g_CoreConfig, "EnableDebugger", 0, "Activate the R4300 debugger when ROM execution begins, if core was built with Debugger support");
     ConfigSetDefaultInt(g_CoreConfig, "CurrentStateSlot", 0, "Save state slot (0-9) to use when saving/loading the emulator state");
     ConfigSetDefaultString(g_CoreConfig, "ScreenshotPath", "", "Path to directory where screenshots are saved. If this is blank, the default value of ${UserConfigPath}/screenshot will be used");
-    ConfigSetDefaultString(g_CoreConfig, "SaveStatePath", "", "Path to directory where save states are saved. If this is blank, the default value of ${UserConfigPath}/save will be used");
+    ConfigSetDefaultString(g_CoreConfig, "SaveStatePath", "", "Path to directory where emulator save states (snapshots) are saved. If this is blank, the default value of ${UserConfigPath}/save will be used");
+    ConfigSetDefaultString(g_CoreConfig, "SaveSRAMPath", "", "Path to directory where SRAM/EEPROM data (in-game saves) are stored. If this is blank, the default value of ${UserConfigPath}/save will be used");
     ConfigSetDefaultString(g_CoreConfig, "SharedDataPath", "", "Path to a directory to search when looking for shared data files");
 
+    /* handle upgrades */
+    if (bUpgrade)
+    {
+        if (fConfigParamsVersion < 1.01f)
+        {  // added separate SaveSRAMPath parameter in v1.01
+            const char *pccSaveStatePath = ConfigGetParamString(g_CoreConfig, "SaveStatePath");
+            if (pccSaveStatePath != NULL)
+                ConfigSetParameter(g_CoreConfig, "SaveSRAMPath", M64TYPE_STRING, pccSaveStatePath);
+        }
+    }
+
+    if (bSaveConfig)
+        ConfigSaveSection("Core");
+
     /* set config parameters for keyboard and joystick commands */
-    event_set_core_defaults();
+    return event_set_core_defaults();
 }
 
 void main_speeddown(int percent)
@@ -196,6 +267,22 @@ void main_speedup(int percent)
     }
 }
 
+void main_speedset(int percent)
+{
+    if (percent < 1 || percent > 1000)
+    {
+        DebugMessage(M64MSG_WARNING, "Invalid speed setting %i percent", percent);
+        return;
+    }
+    // disable fast-forward if it's enabled
+    main_set_fastforward(0);
+    // set speed
+    l_SpeedFactor = percent;
+    main_message(M64MSG_STATUS, OSD_BOTTOM_LEFT, "%s %d%%", "Playback speed:", l_SpeedFactor);
+    setSpeedFactor(l_SpeedFactor);  // call to audio plugin
+    StateChanged(M64CORE_SPEED_FACTOR, l_SpeedFactor);
+}
+
 void main_set_fastforward(int enable)
 {
     static int ff_state = 0;
@@ -211,6 +298,7 @@ void main_set_fastforward(int enable)
         // set fast-forward indicator
         l_msgFF = osd_new_message(OSD_TOP_RIGHT, "Fast Forward");
         osd_message_set_static(l_msgFF);
+        osd_message_set_user_managed(l_msgFF);
     }
     else if (!enable && ff_state)
     {
@@ -225,6 +313,19 @@ void main_set_fastforward(int enable)
 
 }
 
+void main_set_speedlimiter(int enable)
+{
+    if (enable)
+        l_MainSpeedLimit = 1;
+    else
+        l_MainSpeedLimit = 0;
+}
+
+int main_get_speedlimiter(void)
+{
+    return l_MainSpeedLimit ? 1 : 0;
+}
+
 int main_is_paused(void)
 {
     return (g_EmulatorRunning && rompause);
@@ -253,6 +354,7 @@ void main_toggle_pause(void)
         DebugMessage(M64MSG_STATUS, "Emulation paused.");
         l_msgPause = osd_new_message(OSD_MIDDLE_CENTER, "Paused");
         osd_message_set_static(l_msgPause);
+	osd_message_set_user_managed(l_msgPause);
         StateChanged(M64CORE_EMU_STATE, M64EMU_PAUSED);
     }
 
@@ -272,10 +374,6 @@ void main_draw_volume_osd(void)
     char msgString[64];
     const char *volString;
 
-    // if we had a volume message, make sure that it's still in the OSD list, or set it to NULL
-    if (l_msgVol != NULL && !osd_message_valid(l_msgVol))
-        l_msgVol = NULL;
-
     // this calls into the audio plugin
     volString = volumeGetString();
     if (volString == NULL)
@@ -285,22 +383,22 @@ void main_draw_volume_osd(void)
     else
     {
         sprintf(msgString, "%s: %s", "Volume", volString);
-        if (msgString[strlen(msgString) - 1] == '%')
-            strcat(msgString, "%");
     }
 
     // create a new message or update an existing one
     if (l_msgVol != NULL)
-        osd_update_message(l_msgVol, msgString);
-    else
-        l_msgVol = osd_new_message(OSD_MIDDLE_CENTER, msgString);
+        osd_update_message(l_msgVol, "%s", msgString);
+    else {
+        l_msgVol = osd_new_message(OSD_MIDDLE_CENTER, "%s", msgString);
+	osd_message_set_user_managed(l_msgVol);
+    }
 }
 
 /* this function could be called as a result of a keypress, joystick/button movement,
    LIRC command, or 'testshots' command-line option timer */
 void main_take_next_screenshot(void)
 {
-    g_TakeScreenshot = l_CurrentFrame + 1;
+    l_TakeScreenshot = l_CurrentFrame + 1;
 }
 
 void main_state_set_slot(int slot)
@@ -312,6 +410,7 @@ void main_state_set_slot(int slot)
     }
 
     savestates_select_slot(slot);
+    StateChanged(M64CORE_SAVESTATE_SLOT, slot);
 }
 
 void main_state_inc_slot(void)
@@ -378,6 +477,9 @@ m64p_error main_core_state_query(m64p_core_param param, int *rval)
         case M64CORE_SPEED_FACTOR:
             *rval = l_SpeedFactor;
             break;
+        case M64CORE_SPEED_LIMITER:
+            *rval = l_MainSpeedLimit;
+            break;
         default:
             return M64ERR_INPUT_INVALID;
     }
@@ -389,17 +491,24 @@ m64p_error main_core_state_query(m64p_core_param param, int *rval)
 * global functions, callbacks from the r4300 core or from other plugins
 */
 
-void video_plugin_render_callback(void)
+static void video_plugin_render_callback(int bScreenRedrawn)
 {
+    int bOSD = ConfigGetParamBool(g_CoreConfig, "OnScreenDisplay");
+
     // if the flag is set to take a screenshot, then grab it now
-    if (g_TakeScreenshot != 0)
+    if (l_TakeScreenshot != 0)
     {
-        TakeScreenshot(g_TakeScreenshot - 1);  // current frame number +1 is in g_TakeScreenshot
-        g_TakeScreenshot = 0; // reset flag
+        // if the OSD is enabled, and the screen has not been recently redrawn, then we cannot take a screenshot now because
+        // it contains the OSD text.  Wait until the next redraw
+        if (!bOSD || bScreenRedrawn)
+        {
+            TakeScreenshot(l_TakeScreenshot - 1);  // current frame number +1 is in l_TakeScreenshot
+            l_TakeScreenshot = 0; // reset flag
+        }
     }
 
     // if the OSD is enabled, then draw it now
-    if (ConfigGetParamBool(g_CoreConfig, "OnScreenDisplay"))
+    if (bOSD)
     {
         osd_render();
     }
@@ -452,8 +561,9 @@ void new_vi(void)
     {
         CalculatedTime = (unsigned int) (CounterTime + AdjustedLimit * VI_Counter);
         time = (int)(CalculatedTime - CurrentFPSTime);
-        if (time > 0)
+        if (time > 0 && l_MainSpeedLimit)
         {
+            DebugMessage(M64MSG_VERBOSE, "    new_vi(): Waiting %ims", time);
             SDL_Delay(time);
         }
         CurrentFPSTime = CurrentFPSTime + time;
@@ -485,9 +595,6 @@ m64p_error main_run(void)
     savestates_select_slot(ConfigGetParamInt(g_CoreConfig, "CurrentStateSlot"));
     no_compiled_jump = ConfigGetParamBool(g_CoreConfig, "NoCompiledJump");
 
-    /* set up the SDL key repeat and event filter to catch keyboard/joystick commands for the core */
-    event_initialize();
-
     // initialize memory, and do byte-swapping if it's not been done yet
     if (g_MemHasBeenBSwapped == 0)
     {
@@ -502,17 +609,21 @@ m64p_error main_run(void)
     // Attach rom to plugins
     if (!romOpen_gfx())
     {
-        free_memory(); SDL_Quit(); return M64ERR_PLUGIN_FAIL;
+        free_memory(); return M64ERR_PLUGIN_FAIL;
     }
     if (!romOpen_audio())
     {
-        romClosed_gfx(); free_memory(); SDL_Quit(); return M64ERR_PLUGIN_FAIL;
+        romClosed_gfx(); free_memory(); return M64ERR_PLUGIN_FAIL;
     }
     if (!romOpen_input())
     {
-        romClosed_audio(); romClosed_gfx(); free_memory(); SDL_Quit(); return M64ERR_PLUGIN_FAIL;
+        romClosed_audio(); romClosed_gfx(); free_memory(); return M64ERR_PLUGIN_FAIL;
     }
 
+    /* set up the SDL key repeat and event filter to catch keyboard/joystick commands for the core */
+    event_initialize();
+
+    /* initialize the on-screen display */
     if (ConfigGetParamBool(g_CoreConfig, "OnScreenDisplay"))
     {
         // init on-screen display
@@ -544,6 +655,7 @@ m64p_error main_run(void)
     r4300_reset_soft();
     r4300_execute();
 
+    /* now begin to shut down */
 #ifdef WITH_LIRC
     lircStop();
 #endif // WITH_LIRC
@@ -568,8 +680,6 @@ m64p_error main_run(void)
     g_EmulatorRunning = 0;
     StateChanged(M64CORE_EMU_STATE, M64EMU_STOPPED);
 
-    SDL_Quit();
-
     return M64ERR_SUCCESS;
 }
 
@@ -591,6 +701,11 @@ void main_stop(void)
         osd_delete_message(l_msgFF);
         l_msgFF = NULL;
     }
+    if(l_msgVol)
+    {
+        osd_delete_message(l_msgVol);
+        l_msgVol = NULL;
+    }
     if (rompause)
     {
         rompause = 0;
