@@ -24,9 +24,11 @@
 #include <string.h>
 #include <ctype.h>
 
+#define M64P_CORE_PROTOTYPES 1
 #include "api/m64p_types.h"
 #include "api/callbacks.h"
 #include "api/config.h"
+#include "api/m64p_config.h"
 
 #include "md5.h"
 #include "rom.h"
@@ -41,7 +43,7 @@
 
 #define CHUNKSIZE 1024*128 /* Read files 128KB at a time. */
 
-_romdatabase g_romdatabase;
+static _romdatabase g_romdatabase;
 romdatabase_entry empty_entry;
 
 /* Global loaded rom memory space. */
@@ -49,6 +51,8 @@ unsigned char* rom = NULL;
 /* Global loaded rom size. */
 int rom_size = 0;
 
+unsigned char isGoldeneyeRom = 0;
+
 rom_header*       ROM_HEADER;
 m64p_rom_settings ROM_SETTINGS;
 
@@ -95,7 +99,7 @@ static void swap_rom(unsigned char* localrom, unsigned char* imagetype, int load
         for (i = 0; i < loadlength; i+=4)
             {
             temp=localrom[i];
-            localrom[i]=localrom[1+3];
+            localrom[i]=localrom[i+3];
             localrom[i+3]=temp;
             temp=localrom[i+1];
             localrom[i+1]=localrom[i+2];
