@@ -33,31 +33,18 @@ int wstrlen(const wchar_t * s)
 
 namespace OGLFT 
 {
-    // This is the static instance of the FreeType library wrapper ...
-    Library Library::library;
-
-    // ... and this is the FreeType library handle itself.
-    FT_Library Library::library_;
-
-    
-    // The static instance above causes this constructor to be called
-    // when the object module is loaded.
-    Library::Library (void)
+    FT_Library ft_library;
+    bool Init_FT(void)
     {
-        FT_Error error = FT_Init_FreeType(&library_);
+        FT_Error error = FT_Init_FreeType(&ft_library);
         if(error != 0) std::cerr << "[OGLFT] Could not initialize the FreeType library." << std::endl;
+        return (error == 0);
     }
-
-    Library::~Library (void)
+    bool Uninit_FT(void)
     {
-        FT_Error error = FT_Done_FreeType(library_);
+        FT_Error error = FT_Done_FreeType(ft_library);
         if(error != 0) std::cerr << "[OGLFT] Could not terminate the FreeType library." << std::endl;
-    }
-
-    // Return the only instance in the process
-    FT_Library& Library::instance (void)
-    {
-        return library_;
+        return (error == 0);
     }
 
     // Load a new face
@@ -66,7 +53,7 @@ namespace OGLFT
     {
         valid_ = true;
         FT_Face ft_face;
-        FT_Error error = FT_New_Face(Library::instance(), filename, 0, &ft_face);
+        FT_Error error = FT_New_Face(ft_library, filename, 0, &ft_face);
         if(error != 0) 
         {
             valid_ = false;
@@ -139,7 +126,7 @@ namespace OGLFT
     {
         FT_Face ft_face;
 
-        FT_Error error = FT_New_Face(Library::instance(), filename, 0, &ft_face);
+        FT_Error error = FT_New_Face(ft_library, filename, 0, &ft_face);
 
         if(error != 0) return false;
 
