@@ -23,8 +23,11 @@
 
 #include <SDL.h>
 
+#define M64P_CORE_PROTOTYPES 1
 #include "api/m64p_types.h"
 #include "api/callbacks.h"
+#include "api/m64p_vidext.h"
+#include "api/vidext.h"
 #include "memory/memory.h"
 #include "main/rom.h"
 #include "main/main.h"
@@ -44,7 +47,7 @@
 
 unsigned int next_vi;
 int vi_field=0;
-int vi_counter=0;
+static int vi_counter=0;
 typedef struct _interupt_queue
 {
    int type;
@@ -54,7 +57,7 @@ typedef struct _interupt_queue
 
 static interupt_queue *q = NULL;
 
-void clear_queue()
+static void clear_queue(void)
 {
     while(q != NULL)
     {
@@ -64,7 +67,7 @@ void clear_queue()
     }
 }
 
-void print_queue()
+/*static void print_queue(void)
 {
     interupt_queue *aux;
     //if (Count < 0x7000000) return;
@@ -75,11 +78,11 @@ void print_queue()
         DebugMessage(M64MSG_INFO, "Count:%x, %x", (unsigned int)aux->count, aux->type);
         aux = aux->next;
     }
-}
+}*/
 
 static int SPECIAL_done = 0;
 
-int before_event(unsigned int evt1, unsigned int evt2, int type2)
+static int before_event(unsigned int evt1, unsigned int evt2, int type2)
 {
     if(evt1 - Count < 0x80000000)
     {
@@ -174,7 +177,7 @@ void add_interupt_event_count(int type, unsigned int count)
     add_interupt_event(type, (count - Count)/*/2*/);
 }
 
-void remove_interupt_event()
+static void remove_interupt_event(void)
 {
     interupt_queue *aux = q->next;
     if(q->type == SPECIAL_INT) SPECIAL_done = 1;
@@ -199,7 +202,7 @@ unsigned int get_event(int type)
     return 0;
 }
 
-int get_next_event_type()
+int get_next_event_type(void)
 {
     if (q == NULL) return 0;
     return q->type;
@@ -274,7 +277,7 @@ void load_eventqueue_infos(char *buf)
     }
 }
 
-void init_interupt()
+void init_interupt(void)
 {
     SPECIAL_done = 1;
     next_vi = next_interupt = 5000;
@@ -285,7 +288,7 @@ void init_interupt()
     add_interupt_event_count(SPECIAL_INT, 0);
 }
 
-void check_interupt()
+void check_interupt(void)
 {
     if (MI_register.mi_intr_reg & MI_register.mi_intr_mask_reg)
         Cause = (Cause | 0x400) & 0xFFFFFF83;
@@ -313,7 +316,7 @@ void check_interupt()
     }
 }
 
-void gen_interupt()
+void gen_interupt(void)
 {
 
     if (stop == 1)
@@ -380,7 +383,7 @@ void gen_interupt()
             if(rompause)
             {
                 osd_render();  // draw Paused message in case updateScreen didn't do it
-                SDL_GL_SwapBuffers();
+                VidExt_GL_SwapBuffers();
                 while(rompause)
                 {
                     SDL_Delay(10);
@@ -560,10 +563,7 @@ void gen_interupt()
                 {
                     if (blocks[i])
                     {
-                        if (blocks[i]->block) { free_exec(blocks[i]->block); blocks[i]->block = NULL; }
-                        if (blocks[i]->code) { free_exec(blocks[i]->code); blocks[i]->code = NULL; }
-                        if (blocks[i]->jumps_table) { free(blocks[i]->jumps_table); blocks[i]->jumps_table = NULL; }
-                        if (blocks[i]->riprel_table) { free(blocks[i]->riprel_table); blocks[i]->riprel_table = NULL; }
+                        free_block(blocks[i]);
                         free(blocks[i]);
                         blocks[i] = NULL;
                     }
